﻿using MathNet.Numerics.Random;
using PLplot;
using System;
using System.Linq;

namespace ConsoleApp2
{
    class Program
    {
        static void Main(string[] args)
        {
            Func<double, double> f = (x) => (x - 1) * (x - 1);
            Func<double, double> df = (x) => 2 * x - 2;

            // 그래프 출력
            DrawPlotChart(-14, 14, -10, 120, f, df);
        }

        private static void DrawPlotChart(double xMin, double xMax, double yMin, double yMax, 
            Func<double, double> orgDrawFunc, Func<double, double> dfDrawFunc)
        {
            string chartFileName = "click.svg";

            using (var pl = new PLStream())
            {
                pl.sdev("svg");
                pl.sfnam(chartFileName);
                pl.spal0("cmap0_alternate.pal");
                pl.init();

                pl.env(xMin, xMax, yMin, yMax, AxesScale.Independent, AxisBox.BoxTicksLabelsAxes);
                pl.lab("X", "Y", "y = x^2 -2x + 1");

                pl.spal0("");
                pl.col0(PLplot.Color.Blue);

                {
                    double[] ptX = Utils.RangeInclusive(xMin, xMax, 0.01).ToArray();
                    double[] ptY = null;

                    ptY = new double[ptX.Length];
                    for (int i = 0; i < ptX.Length; i++)
                    {
                        ptY[i] = orgDrawFunc(ptX[i]);
                    }

                    pl.line(ptX, ptY);
                }

                char code = Symbol.Bullet;
                pl.col0(PLplot.Color.Blue);

                int maxTrial = 1000;
                double anyX = 15.0;

                while (maxTrial-- > 0)
                {
                    double yPos = dfDrawFunc(anyX);
                    // pl.Point(anyX, orgDrawFunc(anyX), code);

                    if (yPos.GetCloseToZeroSlope())
                    {
                        break;
                    }
                    else anyX += (yPos > 0) ? -0.1 : 0.1;
                }

                pl.col0(PLplot.Color.Red);
                code = Symbol.Star;

                maxTrial = 1000;
                anyX = 15.0;
                double t = 0.1;

                while (maxTrial-- > 0)
                {
                    double yPos = dfDrawFunc(anyX);
                    pl.Point(anyX, orgDrawFunc(anyX), code);

                    if (yPos.GetCloseToZeroSlope())
                    {
                        break;
                    }
                    else anyX -= (t * yPos);
                }

                pl.eop();
                pl.gver(out var verText);
            }
        }
    }
}
